'Copyright (c) 2017,2018 keybonicPlague (Keybone)

'$DYNAMIC
Option _Explicit

_Title "OS/3 Operating Environment"

Dim Shared Mode As String: Mode = "BOOT"
Dim Shared backgroundColor As _Unsigned Long: backgroundColor = _RGBA32(255, 255, 255, 255)
Dim Shared textColor As _Unsigned Long: textColor = _RGBA32(0, 0, 0, 255)
Dim Shared screenResolutionX As _Unsigned Integer: screenResolutionX = 640
Dim Shared screenResolutionY As _Unsigned Integer: screenResolutionY = 480
Dim Shared fullscreenMode As _Bit: fullscreenMode = 0
Dim Shared scaledMode As _Bit: scaledMode = 0
Dim Shared straightToGUI As _Bit: straightToGUI = 0

Type Window
    Handle As _Unsigned _Integer64
    iconHandle As _Unsigned _Integer64 '       : ^ Image handles
    Identifier As _Unsigned _Integer64
    Title As String * 20 '                     : < Window title and icon title
    titleTextSize As _Unsigned _Byte '         : Length of title's text
    previousPositionX As _Unsigned Integer
    previousPositionY As _Unsigned Integer
    positionStartX As Integer
    positionStartY As Integer
    minimumWidth As _Unsigned Integer
    minimumHeight As _Unsigned Integer '       : ^ Minimum size of window (usually startwidth)
    startWidth As _Unsigned Integer
    startHeight As _Unsigned Integer '         : ^ Default size of window
    iconPositionStartX As Integer
    iconPositionStartY As Integer
    iconPositionPreviousX As Integer
    iconPositionPreviousY As Integer
    iconWidth As _Unsigned Integer
    iconHeight As _Unsigned Integer '          : ^ Icon size and position
    iconTitlePositionStartX As Integer
    iconTitlePositionStartY As Integer '       : ^ Icon size and position
    isDialogBox As _Byte
    isMaximizable As _Byte
    isMinimizable As _Byte
    isMovable As _Byte
    isResizable As _Byte '                     : ^ Window attributes
    isMaximized As _Byte
    isRestored As _Byte
    isMinimized As _Byte
    wasMaximized As _Byte
    wasRestored As _Byte
    isActive As _Byte
End Type '                : ^ Window Status

Type Button
    startX As _Unsigned Integer
    startY As _Unsigned Integer
    endX As _Unsigned Integer
    endY As _Unsigned Integer
    typeSelector As _Unsigned _Byte
    isPressed As _Byte
End Type

Dim Shared minimumSize As _Unsigned _Byte: minimumSize = 0
Dim Shared ZORDER(minimumSize) As Window, NULL_Window As Window, zOrderInput As Window
Dim Shared maximumSize As _Unsigned _Integer64, currentSize As _Unsigned _Integer64: maximumSize = 0 - 1: currentSize = minimumSize

Dim Shared previousMouseX As _Unsigned Integer, mouseX As _Unsigned Integer, currentMouseX As _Unsigned Integer
Dim Shared previousMouseY As _Unsigned Integer, mouseY As _Unsigned Integer, currentMouseY As _Unsigned Integer
Dim Shared mouseButton1 As _Byte, mouseButton2 As _Byte, mouseClicked As _Byte

'$include:'default-theme.bi'

Do
    Select Case Mode
        Case "WINDOWS"
            Windows
        Case "COMMAND"
            commandPrompt
        Case "REBOOT"
            Reboot
        Case "BOOT"
            Boot
        Case "QUIT"
            Quit
    End Select
Loop

Sub Boot
    If fullscreenMode Then
        If scaledMode Then
            initializeDisplay screenResolutionX, screenResolutionY, "SCALED"
        Else
            initializeDisplay screenResolutionX, screenResolutionY, "SQUARE"
        End If
    Else
        initializeDisplay screenResolutionX, screenResolutionY, "WINDOW"
    End If
    Color textColor, backgroundColor: Cls
    Print "Booting OS/3...": Print ""
    Sleep 1
    If straightToGUI Then
        Mode = "WINDOWS"
    Else
        Mode = "COMMAND"
    End If
End Sub

Sub Reboot
    cPrint "Rebooting..."
    Sleep 1
    Shell _DontWait "./os3structured-gui"
    System
End Sub

Sub newWindow
    cPrint "Starting new window..."
    Shell _DontWait "./os3structured-gui"
End Sub

Sub initializeDisplay (displayWidth As _Unsigned Integer, displayHeight As _Unsigned Integer, displayOptions As String)
    displayOptions = UCase$(displayOptions)
    If displayWidth And displayHeight Then
        Screen _NewImage(displayWidth, displayHeight, 32)
        _PrintMode _KeepBackground
        Select Case displayOptions
            Case "SQUARE"
                _FullScreen _SquarePixels , _Smooth
            Case "SCALED"
                _FullScreen _Stretch , _Smooth
            Case "WINDOW"
            Case Else
                Print "ERROR: Invalid Option!": Beep
                End
        End Select
    Else
        Print "ERROR: Invalid Dimensions!": Beep
        End
    End If
End Sub

Sub commandPrompt
    Dim cmd As String: Input "> ", cmd
    cmd = UCase$(LTrim$(RTrim$(cmd)))
    Select Case cmd
        Case "WINDOWS"
            Mode = "WINDOWS"
        Case "VERSION"
            Version
        Case "HELP"
            Help
        Case "EXIT"
            Mode = "QUIT"
        Case "REBOOT"
            Mode = "REBOOT"
        Case "NEW WINDOW"
            newWindow
    End Select
End Sub

Sub cPrint (inString As String)
    Print ""
    Print inString
    Print ""
End Sub

Sub Windows
    testObject1: testObject2: testObject4: testObject3
    initializeObjects
    guiMainLoop
    Mode = "COMMAND"
    Cls
End Sub

Sub Version
    cPrint "OS/3 Version 1.0"
End Sub

Sub Quit
    System
End Sub

Sub Help
    cPrint "Commands: windows, version, help, exit, reboot"
End Sub

Sub guiMainLoop
    Dim Iteration As _Unsigned _Integer64
    While currentSize > minimumSize
        placeBackground
        probeMouse
        For Iteration = currentSize To minimumSize + 1 Step -1
            If mouseButton1 Then
                If isWithinWindow(Iteration) Then
                    If ZORDER(Iteration).isMinimized Then
                        moveIcon (Iteration)
                    ElseIf ZORDER(Iteration).isRestored Then
                        raiseToTop (Iteration)
                        If isWithinArea(Iteration, 4, 4, 21, 21) Then
                            removeFromZOrder (Iteration)
                            If Iteration > minimumSize + 1 Then raiseToTop (Iteration - 1)
                            Exit For
                        ElseIf isWithinArea(Iteration, 22, 4, ZORDER(Iteration).startWidth - 42, 21) Then
                            moveWindow (Iteration)
                        ElseIf isWithinArea(Iteration, ZORDER(Iteration).startWidth - 41, 4, ZORDER(Iteration).startWidth - 24, 21) Then
                            If ZORDER(Iteration).isMinimizable Then
                                minimizeWindow (Iteration)
                                If Iteration > minimumSize + 1 Then raiseToTop (Iteration - 1)
                            End If
                        ElseIf isWithinArea(Iteration, ZORDER(Iteration).startWidth - 22, 4, ZORDER(Iteration).startWidth - 5, 21) Then
                            maximizeWindow (Iteration)
                        ElseIf isWithinArea(Iteration, ZORDER(Iteration).startWidth - 22, ZORDER(Iteration).startHeight - 22, ZORDER(Iteration).startWidth - 5, ZORDER(Iteration).startHeight - 5) Then
                            resizeWindow (Iteration)
                        End If
                    ElseIf ZORDER(Iteration).isMaximized Then
                        If isWithinArea(Iteration, 4, 4, 21, 21) Then
                            removeFromZOrder (Iteration)
                            Exit For
                        ElseIf isWithinArea(Iteration, _Width - 41, 4, _Width - 24, 21) Then
                            minimizeWindow (Iteration)
                            If Iteration > minimumSize + 1 Then raiseToTop (Iteration - 1)
                        ElseIf isWithinArea(Iteration, _Width - 22, 4, _Width - 5, 21) Then
                            restoreWindow (Iteration)
                        End If
                    End If
                    Exit For
                End If
            End If
        Next Iteration
        updateDisplay
        _Display
        _Limit 60
    Wend
End Sub

Sub initializeObjects
    Dim I As _Unsigned _Integer64
    For I = currentSize To minimumSize + 1 Step -1
        displayWindow (I)
    Next I
End Sub

Sub initializeBackground (Filename As String, backgroundOptions As String)
    backgroundOptions = UCase$(backgroundOptions)
    If _FileExists(Filename) Then
        Static oldFilename As String
        If Filename <> oldFilename Then
            Static backgroundHandle As Long
            backgroundHandle = _LoadImage(Filename, 32)
        End If
        Select Case backgroundOptions
            Case "RAW"
                _PutImage (0, 0)-(_Width, _Height), backgroundHandle, 0, (0, 0)-(_Width, _Height)
            Case "SCALED"
                _PutImage , backgroundHandle, 0
            Case "TILED"
                Dim tileWidth As _Unsigned Integer: tileWidth = _Width(backgroundHandle)
                Dim tileHeight As _Unsigned Integer: tileHeight = _Height(backgroundHandle)
                Dim numberWide As Single: numberWide = _Width \ tileWidth
                Dim numberHigh As Single: numberHigh = _Height \ tileHeight
                Dim currentDesktopHigh As _Unsigned Integer
                Dim currentDesktopWide As _Unsigned Integer
                For currentDesktopHigh = 0 To numberHigh
                    For currentDesktopWide = 0 To numberWide
                        _PutImage (tileWidth * currentDesktopWide, tileHeight * currentDesktopHigh), backgroundHandle, 0
                    Next currentDesktopWide
                Next currentDesktopHigh
        End Select
        oldFilename = Filename
    Else
        Print "ERROR: Invalid Filename!": Beep
    End If
End Sub

Sub placeBackground
    initializeBackground "ggravity.jpg", "SCALED"
End Sub

Sub probeMouse
    previousMouseX = mouseX
    previousMouseY = mouseY
    Dim i As Integer: While _MouseInput
        i = _MouseInput
    Wend
    mouseButton1 = _MouseButton(1)
    mouseButton2 = _MouseButton(2)
    mouseX = _MouseX: currentMouseX = mouseX
    mouseY = _MouseY: currentMouseY = mouseY
End Sub

Function isWithinWindow%% (Identifier As _Unsigned _Integer64)
    isWithinWindow = 0
    If ZORDER(Identifier).isRestored = -1 Then
        If currentMouseX >= ZORDER(Identifier).positionStartX Then
            If currentMouseY >= ZORDER(Identifier).positionStartY Then
                If currentMouseX <= ZORDER(Identifier).positionStartX + ZORDER(Identifier).startWidth Then
                    If currentMouseY <= ZORDER(Identifier).positionStartY + ZORDER(Identifier).startHeight Then
                        isWithinWindow = -1
                    End If
                End If
            End If
        End If
    ElseIf ZORDER(Identifier).isMaximized = -1 Then
        If currentMouseX >= 0 Then
            If currentMouseY >= 0 Then
                If currentMouseX <= _Width Then
                    If currentMouseY <= _Height Then
                        isWithinWindow = -1
                    End If
                End If
            End If
        End If
    ElseIf ZORDER(Identifier).isMinimized = -1 Then
        If currentMouseX >= ZORDER(Identifier).iconPositionStartX Then
            If currentMouseY >= ZORDER(Identifier).iconPositionStartY Then
                If currentMouseX <= ZORDER(Identifier).iconPositionStartX + ZORDER(Identifier).iconWidth Then
                    If currentMouseY <= ZORDER(Identifier).iconPositionStartY + ZORDER(Identifier).iconHeight Then
                        isWithinWindow = -1
                    End If
                End If
            End If
        End If
    End If
End Function

Function isWithinArea%% (Identifier As _Unsigned _Integer64, startX As _Unsigned Integer, startY As _Unsigned Integer, endX As _Unsigned Integer, endY As _Unsigned Integer)
    If ZORDER(Identifier).isRestored = -1 Then
        If currentMouseX >= (ZORDER(Identifier).positionStartX + startX) Then
            If currentMouseY >= (ZORDER(Identifier).positionStartY + startY) Then
                If currentMouseX <= (ZORDER(Identifier).positionStartX + endX) Then
                    If currentMouseY <= (ZORDER(Identifier).positionStartY + endY) Then
                        isWithinArea = -1
                    End If
                End If
            End If
        End If
    ElseIf ZORDER(Identifier).isMaximized = -1 Then
        If currentMouseX >= startX Then
            If currentMouseY >= startY Then
                If currentMouseX <= endX Then
                    If currentMouseY <= endY Then
                        isWithinArea = -1
                    End If
                End If
            End If
        End If
    End If
End Function

Sub addToZOrder
    If currentSize < maximumSize Then
        currentSize = currentSize + 1
        ReDim _Preserve ZORDER(currentSize) As Window
    End If
    ZORDER(currentSize) = zOrderInput
    zOrderInput = NULL_Window
End Sub

Sub removeFromZOrder (Identifier As _Unsigned _Integer64)
    Dim Iteration As _Unsigned _Integer64
    For Iteration = Identifier To currentSize
        If Iteration < currentSize Then ZORDER(Iteration) = ZORDER(Iteration + 1)
    Next Iteration
    If currentSize > minimumSize Then
        currentSize = currentSize - 1
        ReDim _Preserve ZORDER(currentSize) As Window
    End If
End Sub

Sub raiseToTop (Identifier As _Unsigned _Integer64)
    Dim Temporary As Window
    Dim Iteration As _Integer64
    For Iteration = minimumSize + 1 To currentSize
        ZORDER(Iteration).isActive = 0
        displayWindow (Iteration)
    Next Iteration
    Temporary = ZORDER(Identifier)
    For Iteration = Identifier To currentSize
        If Iteration >= Identifier And Iteration < currentSize Then ZORDER(Iteration) = ZORDER(Iteration + 1)
    Next Iteration
    ZORDER(currentSize) = Temporary
    makeActive
End Sub

Sub makeActive
    Dim Iteration As _Integer64
    For Iteration = minimumSize + 1 To currentSize
        ZORDER(Iteration).isActive = 0
        displayWindow (Iteration)
    Next Iteration
    ZORDER(currentSize).isActive = -1
    displayWindow (currentSize)
End Sub

Sub displayWindow (Identifier As _Unsigned _Integer64)
    _Dest ZORDER(Identifier).Handle
    _PrintMode _KeepBackground
    If ZORDER(Identifier).isActive = -1 Then
        Color whiteColor
    Else
        Color blackColor
    End If
    Dim Centering As _Unsigned Integer
    If ZORDER(Identifier).isMaximized = -1 Then
        Line (0, 0)-(_Width - 1, _Height - 1), blackColor, B
        Line (1, 1)-(_Width - 2, _Height - 2), highlightColor2, B
        Line (2, 2)-(_Width - 2, _Height - 2), shadowColor2, B
        Line (2, 2)-(_Width - 3, _Height - 3), highlightColor, B
        Line (3, 3)-(_Width - 3, _Height - 3), shadowColor, B
        Line (3, 3)-(_Width - 4, _Height - 4), blackColor, B
        Line (0, _Height - 23)-(3, _Height - 23), blackColor
        Line (23, _Height - 3)-(23, _Height), blackColor
        Line (_Width, _Height - 23)-(_Width - 3, _Height - 23), blackColor
        Line (_Width - 23, _Height - 3)-(_Width - 23, _Height), blackColor
        Line (4, 4)-(_Width - 5, _Height - 5), windowBackgroundColor, BF
        If ZORDER(Identifier).isActive = -1 Then
            Line (4, 4)-(_Width - 6, 21), activeTitlebarColor, BF
        Else
            Line (4, 4)-(_Width - 6, 21), inactiveTitlebarColor, BF
        End If
        Line (_Width - 23, 0)-(_Width - 1, 22), blackColor, B
        Line (_Width - 42, 3)-(_Width - 23, 22), blackColor, B
        Line (0, 0)-(22, 22), blackColor, B
        Line (0, 22)-(_Width - 1, 22), blackColor
        Call displayButton(4, 4, 21, 21, 5, 0)
        Call displayButton(_Width - 41, 4, _Width - 24, 21, 3, 0)
        Centering = (_Width - (60 + ZORDER(Identifier).titleTextSize)) / 2
        _PrintString (Centering + 20, 5), ZORDER(Identifier).Title
        If ZORDER(Identifier).isMaximized = 0 Then
            Call displayButton(_Width - 22, 4, _Width - 5, 21, 1, 0)
        ElseIf ZORDER(Identifier).isMaximized = -1 Then
            Call displayButton(_Width - 22, 4, _Width - 5, 21, 2, 0)
        End If
    ElseIf ZORDER(Identifier).isRestored = -1 Then
        Line (0, 0)-(ZORDER(Identifier).startWidth - 1, ZORDER(Identifier).startHeight - 1), blackColor, B
        Line (1, 1)-(ZORDER(Identifier).startWidth - 2, ZORDER(Identifier).startHeight - 2), highlightColor2, B
        Line (2, 2)-(ZORDER(Identifier).startWidth - 2, ZORDER(Identifier).startHeight - 2), shadowColor2, B
        Line (2, 2)-(ZORDER(Identifier).startWidth - 3, ZORDER(Identifier).startHeight - 3), highlightColor, B
        Line (3, 3)-(ZORDER(Identifier).startWidth - 3, ZORDER(Identifier).startHeight - 3), shadowColor, B
        Line (3, 3)-(ZORDER(Identifier).startWidth - 4, ZORDER(Identifier).startHeight - 4), blackColor, B
        Line (0, ZORDER(Identifier).startHeight - 23)-(3, ZORDER(Identifier).startHeight - 23), blackColor
        Line (23, ZORDER(Identifier).startHeight - 3)-(23, ZORDER(Identifier).startHeight), blackColor
        If ZORDER(Identifier).isResizable = 0 Then
            Line (ZORDER(Identifier).startWidth, ZORDER(Identifier).startHeight - 23)-(ZORDER(Identifier).startWidth - 3, ZORDER(Identifier).startHeight - 23), blackColor
            Line (ZORDER(Identifier).startWidth - 23, ZORDER(Identifier).startHeight - 3)-(ZORDER(Identifier).startWidth - 23, ZORDER(Identifier).startHeight), blackColor
        End If
        Line (4, 4)-(ZORDER(Identifier).startWidth - 5, ZORDER(Identifier).startHeight - 5), windowBackgroundColor, BF
        If ZORDER(Identifier).isActive = -1 Then
            Line (4, 4)-(ZORDER(Identifier).startWidth - 6, 21), activeTitlebarColor, BF
        Else
            Line (4, 4)-(ZORDER(Identifier).startWidth - 6, 21), inactiveTitlebarColor, BF
        End If
        Line (ZORDER(Identifier).startWidth - 23, 0)-(ZORDER(Identifier).startWidth - 1, 22), blackColor, B
        Line (ZORDER(Identifier).startWidth - 42, 3)-(ZORDER(Identifier).startWidth - 23, 22), blackColor, B
        Line (0, 0)-(22, 22), blackColor, B
        Line (0, 22)-(ZORDER(Identifier).startWidth - 1, 22), blackColor
        Call displayButton(4, 4, 21, 21, 5, 0)
        Call displayButton(ZORDER(Identifier).startWidth - 41, 4, ZORDER(Identifier).startWidth - 24, 21, 3, 0)
        Centering = (ZORDER(Identifier).startWidth - (60 + ZORDER(Identifier).titleTextSize)) / 2
        _PrintString (Centering + 20, 5), ZORDER(Identifier).Title
        If ZORDER(Identifier).isMaximized = 0 Then
            Call displayButton(ZORDER(Identifier).startWidth - 22, 4, ZORDER(Identifier).startWidth - 5, 21, 1, 0)
        ElseIf ZORDER(Identifier).isMaximized = -1 Then
            Call displayButton(ZORDER(Identifier).startWidth - 22, 4, ZORDER(Identifier).startWidth - 5, 21, 2, 0)
        End If
        If ZORDER(Identifier).isResizable = -1 Then
            Line (ZORDER(Identifier).startWidth - 23, ZORDER(Identifier).startHeight - 23)-(ZORDER(Identifier).startWidth - 1, ZORDER(Identifier).startHeight - 1), blackColor, B
            Call displayButton(ZORDER(Identifier).startWidth - 22, ZORDER(Identifier).startHeight - 22, ZORDER(Identifier).startWidth - 5, ZORDER(Identifier).startHeight - 5, 4, 0)
        End If
    ElseIf ZORDER(Identifier).isDialogBox = -1 Then
        Line (0, 0)-(ZORDER(Identifier).startWidth - 1, ZORDER(Identifier).startHeight - 1), blackColor, B
        Line (1, 1)-(ZORDER(Identifier).startWidth - 2, ZORDER(Identifier).startHeight - 2), windowBackgroundColor, BF
        If ZORDER(Identifier).isActive Then Line (1, 1)-(ZORDER(Identifier).startWidth - 2, ZORDER(Identifier).startHeight - 2), activeTitlebarColor, BF
        Line (4, 4)-(ZORDER(Identifier).startWidth - 5, ZORDER(Identifier).startHeight - 5), windowBackgroundColor, BF
        Call displayButton(5, 5, 22, 22, 6, 0)
        Line (23, 5)-(23, 23), blackColor
        Line (5, 23)-(23, 23), blackColor
        If ZORDER(Identifier).isActive = -1 Then Line (24, 5)-(ZORDER(Identifier).startWidth - 6, 23), activeTitlebarColor, BF
        Line (5, 23)-(ZORDER(Identifier).startWidth - 6, 23), blackColor
        Centering = (ZORDER(Identifier).startWidth - (30 + ZORDER(Identifier).titleTextSize)) / 2
        _PrintString (Centering + 23, 7), ZORDER(Identifier).Title
    End If
    _Dest 0
End Sub

Sub displayButton (buttonStartX As _Unsigned Integer, buttonStartY As _Unsigned Integer, buttonEndX As _Unsigned Integer, buttonEndY As _Unsigned Integer, buttonTypeSelector As _Unsigned _Byte, buttonPressed As _Byte)
    If buttonTypeSelector <= 4 Then
        If buttonPressed = 0 Then
            Line (buttonStartX, buttonStartY)-(buttonEndX, buttonEndY), shadowColor2, B
            Line (buttonStartX, buttonStartY)-(buttonEndX - 1, buttonEndY - 1), highlightColor, B
            Line (buttonStartX + 1, buttonStartY + 1)-(buttonEndX - 1, buttonEndY - 1), shadowColor2, B
            Line (buttonStartX + 1, buttonStartY + 1)-(buttonEndX - 2, buttonEndY - 2), highlightColor, B
            Line (buttonStartX + 2, buttonStartY + 2)-(buttonEndX - 2, buttonEndY - 2), faceColor, BF
        ElseIf buttonPressed = -1 Then
            Line (buttonStartX, buttonStartY)-(buttonEndX, buttonEndY), highlightColor, B
            Line (buttonStartX, buttonStartY)-(buttonEndX - 1, buttonEndY - 1), shadowColor2, B
            Line (buttonStartX + 1, buttonStartY + 1)-(buttonEndX - 1, buttonEndY - 1), highlightColor, B
            Line (buttonStartX + 1, buttonStartY + 1)-(buttonEndX - 2, buttonEndY - 2), shadowColor2, B
            Line (buttonStartX + 2, buttonStartY + 2)-(buttonEndX - 2, buttonEndY - 2), faceColor, BF
        End If
    End If
    Select Case buttonTypeSelector
        Case Is = 0
        Case Is = 1
            Line (buttonStartX + 8, buttonStartY + 6)-(buttonStartX + 8, buttonStartY + 6), blackColor
            Line (buttonStartX + 7, buttonStartY + 7)-(buttonStartX + 9, buttonStartY + 7), blackColor
            Line (buttonStartX + 6, buttonStartY + 8)-(buttonStartX + 10, buttonStartY + 8), blackColor
            Line (buttonStartX + 5, buttonStartY + 9)-(buttonStartX + 11, buttonStartY + 9), blackColor
        Case Is = 2
            Line (buttonStartX + 8, buttonStartY + 4)-(buttonStartX + 8, buttonStartY + 4), blackColor
            Line (buttonStartX + 7, buttonStartY + 5)-(buttonStartX + 9, buttonStartY + 5), blackColor
            Line (buttonStartX + 6, buttonStartY + 6)-(buttonStartX + 10, buttonStartY + 6), blackColor
            Line (buttonStartX + 5, buttonStartY + 7)-(buttonStartX + 11, buttonStartY + 7), blackColor
            Line (buttonStartX + 5, buttonStartY + 10)-(buttonStartX + 11, buttonStartY + 10), blackColor
            Line (buttonStartX + 6, buttonStartY + 11)-(buttonStartX + 10, buttonStartY + 11), blackColor
            Line (buttonStartX + 7, buttonStartY + 12)-(buttonStartX + 9, buttonStartY + 12), blackColor
            Line (buttonStartX + 8, buttonStartY + 13)-(buttonStartX + 8, buttonStartY + 13), blackColor
        Case Is = 3
            Line (buttonStartX + 5, buttonStartY + 7)-(buttonStartX + 11, buttonStartY + 7), blackColor
            Line (buttonStartX + 6, buttonStartY + 8)-(buttonStartX + 10, buttonStartY + 8), blackColor
            Line (buttonStartX + 7, buttonStartY + 9)-(buttonStartX + 9, buttonStartY + 9), blackColor
            Line (buttonStartX + 8, buttonStartY + 10)-(buttonStartX + 8, buttonStartY + 10), blackColor
        Case Is = 4
            Line (buttonStartX + 4, buttonStartY + 12)-(buttonStartX + 12, buttonStartY + 12), blackColor
            Line (buttonStartX + 12, buttonStartY + 4)-(buttonStartX + 12, buttonStartY + 12), blackColor
            Line (buttonStartX + 4, buttonStartY + 10)-(buttonStartX + 10, buttonStartY + 10), blackColor
            Line (buttonStartX + 10, buttonStartY + 4)-(buttonStartX + 10, buttonStartY + 10), blackColor
        Case Is = 5
            Line (buttonStartX, buttonStartY)-(buttonEndX, buttonEndY), faceColor, BF
            Line (buttonStartX + 3, buttonStartY + 9)-(buttonStartX + 16, buttonStartY + 11), shadowColor, B
            Line (buttonStartX + 2, buttonStartY + 8)-(buttonStartX + 15, buttonStartY + 10), blackColor, B
            Line (buttonStartX + 3, buttonStartY + 9)-(buttonStartX + 14, buttonStartY + 9), whiteColor
        Case Is = 6
            Line (buttonStartX, buttonStartY)-(buttonEndX, buttonEndY), faceColor, BF
            Line (buttonStartX + 6, buttonStartY + 9)-(buttonStartX + 11, buttonStartY + 11), shadowColor, B
            Line (buttonStartX + 5, buttonStartY + 8)-(buttonStartX + 12, buttonStartY + 10), blackColor, B
            Line (buttonStartX + 6, buttonStartY + 9)-(buttonStartX + 11, buttonStartY + 9), whiteColor
    End Select
End Sub

Sub updateDisplay
    Dim Iteration As _Unsigned _Integer64
    Dim Temporary As _Unsigned Integer
    For Iteration = minimumSize + 1 To currentSize
        If ZORDER(Iteration).isMinimized = -1 Then
            _PutImage (ZORDER(Iteration).iconPositionStartX, ZORDER(Iteration).iconPositionStartY), ZORDER(Iteration).iconHandle
            Temporary = (ZORDER(Iteration).titleTextSize - ZORDER(Iteration).iconWidth) / 2
            ZORDER(Iteration).iconTitlePositionStartX = (ZORDER(Iteration).iconPositionStartX - Temporary + 2)
            ZORDER(Iteration).iconTitlePositionStartY = ((ZORDER(Iteration).iconPositionStartY + ZORDER(Iteration).iconHeight) + 6)
            Line ((ZORDER(Iteration).iconPositionStartX - Temporary), (ZORDER(Iteration).iconPositionStartY + ZORDER(Iteration).iconHeight) + 5)-(((ZORDER(Iteration).iconPositionStartX - Temporary) + (ZORDER(Iteration).titleTextSize) + 2), (ZORDER(Iteration).iconPositionStartY + ZORDER(Iteration).iconHeight) + 20), whiteColor, BF
            Line ((ZORDER(Iteration).iconPositionStartX - Temporary), (ZORDER(Iteration).iconPositionStartY + ZORDER(Iteration).iconHeight) + 5)-(((ZORDER(Iteration).iconPositionStartX - Temporary) + (ZORDER(Iteration).titleTextSize) + 2), (ZORDER(Iteration).iconPositionStartY + ZORDER(Iteration).iconHeight) + 20), blackColor, B
            Line ((ZORDER(Iteration).iconPositionStartX - Temporary), (ZORDER(Iteration).iconPositionStartY + ZORDER(Iteration).iconHeight) + 5)-(((ZORDER(Iteration).iconPositionStartX - Temporary) + (ZORDER(Iteration).titleTextSize) + 2), (ZORDER(Iteration).iconPositionStartY + ZORDER(Iteration).iconHeight) + 20), activeTitlebarColor, BF
            Color blackColor
            _PrintString (ZORDER(Iteration).iconTitlePositionStartX, ZORDER(Iteration).iconTitlePositionStartY), ZORDER(Iteration).Title
        ElseIf ZORDER(Iteration).isRestored = -1 Then
            Line (ZORDER(Iteration).positionStartX + shadowDepth, ZORDER(Iteration).positionStartY + shadowDepth)-((ZORDER(Iteration).positionStartX + ZORDER(Iteration).startWidth) + shadowDepth, (ZORDER(Iteration).positionStartY + ZORDER(Iteration).startHeight) + shadowDepth), windowShadowColor, BF
            _PutImage (ZORDER(Iteration).positionStartX, ZORDER(Iteration).positionStartY), ZORDER(Iteration).Handle
        ElseIf ZORDER(Iteration).isMaximized = -1 Then
            _PutImage (0, 0), ZORDER(Iteration).Handle
        End If
    Next Iteration
End Sub

Sub maximizeWindow (Identifier As _Unsigned _Integer64)
    If ZORDER(Identifier).isMaximizable = -1 Then
        ZORDER(Identifier).isMinimized = 0
        ZORDER(Identifier).isRestored = 0
        ZORDER(Identifier).isMaximized = -1
        Close ZORDER(Identifier).Handle
        ZORDER(Identifier).Handle = _NewImage(_Width, _Height, 32)
        displayWindow (Identifier)
    End If
End Sub

Sub minimizeWindow (Identifier As _Unsigned _Integer64)
    If ZORDER(Identifier).isRestored = -1 Then
        ZORDER(Identifier).wasRestored = -1
        ZORDER(Identifier).isRestored = 0
    ElseIf ZORDER(Identifier).isMaximized = -1 Then
        ZORDER(Identifier).wasMaximized = -1
        ZORDER(Identifier).isMaximized = 0
    End If
    ZORDER(Identifier).isMinimized = -1
End Sub

Sub restoreWindow (Identifier As _Unsigned _Integer64)
    If ZORDER(Identifier).isMaximized = -1 Then
        ZORDER(Identifier).isMaximized = 0
        ZORDER(Identifier).isRestored = -1
        Close ZORDER(Identifier).Handle
        ZORDER(Identifier).Handle = _NewImage(ZORDER(Identifier).startWidth, ZORDER(Identifier).startHeight, 32)
        displayWindow (Identifier)
    Else
        If ZORDER(Identifier).isMinimized = -1 And ZORDER(Identifier).wasRestored = -1 Then
            ZORDER(Identifier).wasRestored = 0
            ZORDER(Identifier).isMinimized = 0
            ZORDER(Identifier).isRestored = -1
        ElseIf ZORDER(Identifier).isMinimized = -1 And ZORDER(Identifier).wasMaximized = -1 Then
            ZORDER(Identifier).wasMaximized = 0
            maximizeWindow (Identifier)
        End If
    End If
    raiseToTop (Identifier)
End Sub

Sub moveWindow (Identifier As _Unsigned _Integer64)
    If ZORDER(Identifier).isMovable = -1 Then
        If ZORDER(Identifier).isRestored = -1 And ZORDER(Identifier).isMinimized = 0 Then
            ZORDER(Identifier).previousPositionX = (ZORDER(Identifier).positionStartX - mouseX)
            ZORDER(Identifier).previousPositionY = (ZORDER(Identifier).positionStartY - mouseY)
            While mouseButton1
                placeBackground
                probeMouse
                ZORDER(Identifier).positionStartX = (mouseX + ZORDER(Identifier).previousPositionX)
                ZORDER(Identifier).positionStartY = (mouseY + ZORDER(Identifier).previousPositionY)
                updateDisplay
                _Display
            Wend
        End If
    End If
End Sub

Sub moveIcon (identifier As _Unsigned _Integer64)
    If ZORDER(identifier).isMinimized = -1 And ZORDER(identifier).isRestored = 0 Then
        Dim temporaryIconStartX As _Unsigned Integer, temporaryIconStartY As _Unsigned Integer
        temporaryIconStartX = ZORDER(identifier).iconPositionStartX
        temporaryIconStartY = ZORDER(identifier).iconPositionStartY
        ZORDER(identifier).iconPositionPreviousX = (ZORDER(identifier).iconPositionStartX - mouseX)
        ZORDER(identifier).iconPositionPreviousY = (ZORDER(identifier).iconPositionStartY - mouseY)
        While mouseButton1
            probeMouse
            If currentMouseX <> previousMouseX Or currentMouseY <> previousMouseY Then
                ZORDER(identifier).iconPositionStartX = (mouseX + ZORDER(identifier).iconPositionPreviousX)
                ZORDER(identifier).iconPositionStartY = (mouseY + ZORDER(identifier).iconPositionPreviousY)
                placeBackground
                updateDisplay
                _Display
            End If
        Wend
        If temporaryIconStartX = ZORDER(identifier).iconPositionStartX Then
            If temporaryIconStartY = ZORDER(identifier).iconPositionStartY Then
                restoreWindow (identifier)
            End If
        End If
    End If
End Sub

Sub resizeWindow (Identifier As _Unsigned _Integer64)
    If ZORDER(Identifier).isResizable = -1 Then
        Dim resizeWidth As _Unsigned Integer, resizeHeight As _Unsigned Integer
        resizeWidth = ZORDER(Identifier).startWidth
        resizeHeight = ZORDER(Identifier).startHeight
        While mouseButton1
            placeBackground
            probeMouse
            resizeWidth = (mouseX - ZORDER(Identifier).positionStartX)
            resizeHeight = (mouseY - ZORDER(Identifier).positionStartY)
            updateDisplay
            Line (ZORDER(Identifier).positionStartX, ZORDER(Identifier).positionStartY)-Step(resizeWidth, resizeHeight), resizeBoxColor, B
            _Display
        Wend
        ZORDER(Identifier).startWidth = resizeWidth
        ZORDER(Identifier).startHeight = resizeHeight
        _FreeImage ZORDER(Identifier).Handle
        ZORDER(Identifier).Handle = _NewImage(ZORDER(Identifier).startWidth + 1, ZORDER(Identifier).startHeight + 1, 32)
        displayWindow (Identifier)
    End If
End Sub

Sub testObject1
    zOrderInput.Identifier = 1
    zOrderInput.Title = "Exit OS/3"
    zOrderInput.titleTextSize = _PrintWidth(RTrim$(LTrim$(zOrderInput.Title)))
    zOrderInput.minimumWidth = 360
    zOrderInput.minimumHeight = 200
    zOrderInput.startWidth = zOrderInput.minimumWidth
    zOrderInput.startHeight = zOrderInput.minimumHeight
    zOrderInput.positionStartX = 200
    zOrderInput.positionStartY = 200
    zOrderInput.iconWidth = 32
    zOrderInput.iconHeight = 32
    zOrderInput.iconPositionStartX = 100
    zOrderInput.iconPositionStartY = (_Height - 100)
    zOrderInput.isDialogBox = -1
    zOrderInput.isMaximizable = -1
    zOrderInput.isMinimizable = -1
    zOrderInput.isMovable = -1
    zOrderInput.isResizable = -1
    zOrderInput.isMaximized = 0
    zOrderInput.isRestored = 0
    zOrderInput.wasMaximized = 0
    zOrderInput.wasRestored = -1
    zOrderInput.isMinimized = -1
    zOrderInput.isActive = 0
    addToZOrder
    zOrderInput = NULL_Window
    ZORDER(currentSize).Handle = _NewImage(ZORDER(currentSize).startWidth, ZORDER(currentSize).startHeight, 32)
    ZORDER(currentSize).iconHandle = _LoadImage("./exit.png")
End Sub

Sub testObject2
    zOrderInput.Identifier = 2
    zOrderInput.Title = "Mine"
    zOrderInput.titleTextSize = _PrintWidth(RTrim$(LTrim$(zOrderInput.Title)))
    zOrderInput.minimumWidth = 150
    zOrderInput.minimumHeight = 170
    zOrderInput.startWidth = 160
    zOrderInput.startHeight = 180
    zOrderInput.positionStartX = 200
    zOrderInput.positionStartY = 200
    zOrderInput.iconWidth = 32
    zOrderInput.iconHeight = 32
    zOrderInput.iconPositionStartX = 200
    zOrderInput.iconPositionStartY = (_Height - 100)
    zOrderInput.isDialogBox = 0
    zOrderInput.isMaximizable = -1
    zOrderInput.isMinimizable = -1
    zOrderInput.isMovable = -1
    zOrderInput.isResizable = -1
    zOrderInput.isMaximized = 0
    zOrderInput.isRestored = 0
    zOrderInput.wasMaximized = 0
    zOrderInput.wasRestored = -1
    zOrderInput.isMinimized = -1
    zOrderInput.isActive = 0
    addToZOrder
    zOrderInput = NULL_Window
    ZORDER(currentSize).Handle = _NewImage(ZORDER(currentSize).startWidth, ZORDER(currentSize).startHeight, 32)
    ZORDER(currentSize).iconHandle = _LoadImage("./mines.png")
End Sub

Sub testObject3
    zOrderInput.Identifier = 3
    zOrderInput.Title = "About"
    zOrderInput.titleTextSize = _PrintWidth(RTrim$(LTrim$(zOrderInput.Title)))
    zOrderInput.minimumWidth = 0
    zOrderInput.minimumHeight = 0
    zOrderInput.startWidth = 340
    zOrderInput.startHeight = 200
    zOrderInput.positionStartX = 100
    zOrderInput.positionStartY = 150
    zOrderInput.iconWidth = 32
    zOrderInput.iconHeight = 32
    zOrderInput.iconPositionStartX = 300
    zOrderInput.iconPositionStartY = (_Height - 100)
    zOrderInput.isDialogBox = -1
    zOrderInput.isMaximizable = 0
    zOrderInput.isMinimizable = -1
    zOrderInput.isMovable = -1
    zOrderInput.isResizable = 0
    zOrderInput.isMaximized = 0
    zOrderInput.isRestored = 0
    zOrderInput.wasMaximized = 0
    zOrderInput.wasRestored = -1
    zOrderInput.isMinimized = -1
    zOrderInput.isActive = 0
    addToZOrder: zOrderInput = NULL_Window
    ZORDER(currentSize).Handle = _NewImage(ZORDER(currentSize).startWidth, ZORDER(currentSize).startHeight, 32)
    ZORDER(currentSize).iconHandle = _LoadImage("./progman.png")
End Sub

Sub testObject4
    zOrderInput.Identifier = 4
    zOrderInput.Title = "Display"
    zOrderInput.titleTextSize = _PrintWidth(RTrim$(LTrim$(zOrderInput.Title)))
    zOrderInput.minimumWidth = 0
    zOrderInput.minimumHeight = 0
    zOrderInput.startWidth = 205
    zOrderInput.startHeight = 285
    zOrderInput.positionStartX = 200
    zOrderInput.positionStartY = 200
    zOrderInput.iconWidth = 32
    zOrderInput.iconHeight = 32
    zOrderInput.iconPositionStartX = 400
    zOrderInput.iconPositionStartY = (_Height - 100)
    zOrderInput.isDialogBox = 0
    zOrderInput.isMaximizable = -1
    zOrderInput.isMinimizable = -1
    zOrderInput.isMovable = -1
    zOrderInput.isResizable = -1
    zOrderInput.isMaximized = 0
    zOrderInput.isRestored = 0
    zOrderInput.wasMaximized = 0
    zOrderInput.wasRestored = -1
    zOrderInput.isMinimized = -1
    zOrderInput.isActive = 0
    addToZOrder
    zOrderInput = NULL_Window
    ZORDER(currentSize).Handle = _NewImage(ZORDER(currentSize).startWidth, ZORDER(currentSize).startHeight, 32)
    ZORDER(currentSize).iconHandle = _LoadImage("./displayprop.png")
End Sub
